use crate::schema::images;

use anyhow::{Context, Result};
use chrono::NaiveDateTime;
use std::ffi::OsString;
use std::{
    fs,
    path::{Path, PathBuf},
    time::{SystemTime, UNIX_EPOCH},
};
use uuid::Uuid;

#[derive(Debug, Clone, Queryable, Insertable, Identifiable)]
pub struct Image {
    pub id: String,
    pub path: String,
    pub title: Option<String>,
    pub last_changed: NaiveDateTime,
    pub published: bool,
}

impl Image {
    pub fn try_from(path: &Path) -> anyhow::Result<Self> {
        let abs_path = path.canonicalize().context("Path not valid!")?;
        let os_path = abs_path.clone().into_os_string();
        let path_str = os_path.into_string().expect(&format!(
            "Converting path-name problem! {:?}",
            path.display()
        ));
        let uuid = Uuid::new_v5(&Uuid::NAMESPACE_URL, &path_str.as_bytes());
        let metadata = abs_path
            .metadata()
            .context(format!("Unable to load Metadata for: {}", path.display()))?;
        let time = match metadata.modified() {
            Ok(t) => t,
            Err(_) => metadata.created().context(format!(
                "Unable to read created-timestamp of: {}",
                path.display()
            ))?,
        };
        let mtime = time.duration_since(UNIX_EPOCH).unwrap();
        Ok(Image {
            id: uuid.to_string(),
            path: path_str.to_string(),
            title: None,
            last_changed: NaiveDateTime::from_timestamp(
                mtime.as_secs() as i64,
                mtime.subsec_nanos() as u32,
            ),
            published: false,
        })
    }
}
